package com.example.tencentnavigation.tencentnavidemo;

import android.content.Context;
import android.graphics.Bitmap;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;


import com.tencent.map.navi.INaviView;
import com.tencent.map.navi.data.GuidedLaneInfo;
import com.tencent.map.navi.data.NavigationData;
import com.tencent.map.navi.data.RecommendRouteInfo;
import com.tencent.map.navi.data.RouteChangedReason;
import com.tencent.map.navi.data.RouteData;
import com.tencent.map.navi.data.RouteTrafficStatus;
import com.tencent.map.navi.data.TrafficItem;
import com.tencent.tencentmap.mapsdk.maps.model.LatLng;

import java.util.ArrayList;
import java.util.Locale;

public class CustomNavView extends FrameLayout implements INaviView {

    private ImageView turnIcon;
    private TextView distanceInfo;
    private ImageView guideLane;


    public CustomNavView(Context context) {
        super(context);
        init();
    }

    public CustomNavView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public CustomNavView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {
        LayoutInflater.from(getContext()).inflate(R.layout.layout_custom_view, this);
        turnIcon = findViewById(R.id.turn_icon);
        distanceInfo = findViewById(R.id.distance_info);
        guideLane = findViewById(R.id.guide_lane);
        setVisibility(View.GONE);
    }

    @Override
    public void onUpdateNavigationData(NavigationData navigationData) {
        if (navigationData == null) {
            return;
        }
        turnIcon.setImageBitmap(navigationData.getTurnIcon());

        String text;
        int distance = navigationData.getLeftDistance();
        if (distance > 1000) {
            text = String.format(Locale.getDefault(), "%.1f公里", distance / 1000f);
        } else if(distance <= 0){
            text = "现在";
        } else {
            text = String.format(Locale.getDefault(), "%d米", distance);
        }
        distanceInfo.setText(text);
    }

    @Override
    public void onShowEnlargedIntersection(Bitmap bitmap) {

    }

    @Override
    public void onHideEnlargedIntersection() {

    }

    @Override
    public void onShowGuidedLane(Bitmap bitmap) {
        guideLane.setImageBitmap(bitmap);
        guideLane.setVisibility(View.VISIBLE);
    }

    @Override
    public void onShowGuidedLaneInfo(GuidedLaneInfo guidedLaneInfo) {

    }

    @Override
    public void onHideGuidedLane() {
        guideLane.setImageBitmap(null);
        guideLane.setVisibility(View.GONE);
    }

    @Override
    public void onSmartLocStart() {

    }

    @Override
    public void onSmartLocEnd() {

    }

    @Override
    public void onGpsRssiChanged(int i) {

    }

    @Override
    public void onGpsWeakNotify() {

    }

    @Override
    public void onGpsStrongNotify() {

    }

    @Override
    public void onRouteDidChange(RouteData routeData, ArrayList<TrafficItem> arrayList) {

    }

    @Override
    public void onRouteDidChange(RouteData routeData, ArrayList<TrafficItem> arrayList, RouteChangedReason routeChangedReason) {

    }

    @Override
    public void onDeleteBackupRoutes(ArrayList<String> arrayList) {

    }

    @Override
    public void onAddBackupRoutes(ArrayList<RouteData> arrayList) {

    }

    @Override
    public void onUpdateTraffic(String s, int i, int i1, ArrayList<LatLng> arrayList, ArrayList<TrafficItem> arrayList1, boolean b) {

    }

    @Override
    public void onUpdateTraffic(RouteTrafficStatus routeTrafficStatus) {

    }

    @Override
    public void onUpdateBackupRoutesTraffic(ArrayList<RouteTrafficStatus> arrayList) {

    }

    @Override
    public void onRouteRecommend(RecommendRouteInfo recommendRouteInfo) {

    }
}
